unit tidyfuncs;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface
uses Windows, Classes, SysUtils;

const
  CRL2 = #13#10#13#10; // two carriage-return linefeed pairs
  sAAWarn = 'WinTidy has detected that the desktop is configured '+
    'to auto-arrange icons. This setting will prevent WinTidy from '+
    'doing its job. To change this setting, right-click the '+
    'desktop, choose "Arrange Icons" from the popup menu, and '+
    'click "Auto Arrange" to remove the check next to it.'+CRL2+
    'Please make this change now.'+CRL2+'Click OK when you '+
    'have corrected the setting, or click Cancel to cancel '+
    'loading WinTidy.';
  sTurnoff = 'To turn off Active Desktop, right-click the desktop, '+
    'click "Active Desktop" in the popup menu, and remove the '+
    'check next to "View As Web Page" or "Show Web Content".';
  sADWarn1 = 'The Windows "Active Desktop" feature is enabled on '+
    'your system. WinTidy''s ability to put the desktop in Small '+
    'icons mode will be disabled, as it is not compatible with '+
    'Active Desktop. If you want to take advantage of Small icons '+
    'mode, you must turn off Active Desktop.'+CRL2+sTurnoff;
  sADWarn2 = 'Please note that WinTidy''s ability to switch the '+
    'desktop into Small icons mode is not compatible with the '+
    'Windows "Active Desktop" feature. This is not a problem at '+
    'present, because Active Desktop is not turned on.'+CRL2+
    'You must NOT turn Active Desktop on while the desktop is in '+
    'Small icons mode. Doing so can cause all icons to disappear '+
    'from your desktop. If this does happen, you can recover your '+
    'desktop icons by logging out and logging in again.'+CRL2+
    'Switch to Small icons mode anyway?';
  sADWarn3 = 'WinTidy''s Small icons mode is not compatible with '+
    'the Windows "Active Desktop" feature, which is turned on at '+
    'present.'+CRL2+sTurnoff;
  sADWarn4 = 'Note: This layout was saved under WinTidy''s Small '+
    'icons mode. Small icons mode is incompatible with the '+
    'Windows Active Desktop feature, which is turned on at '+
    'present. The layout has been restored in normal icons mode. '+
    'You may need to reposition the icons.'+CRL2+sTurnoff;
  sFmtManyIcons = 'The desktop contains %d icons, and the layout '+
    'can only accommodate %d. Please %s or remove unnecessary icons '+
    'and try again.';
  sFmtRemains = 'The restored %s layout does not include '+
    'positioning information for the following icon or icons:'+CRL2+
    '%s'+CRL2+'We recommend that you place these icons where you '+
    'wish and then re-save the layout.';
  sFmtDupes = 'Your desktop has more than one icon with the '+
    'following name or names:'+CRL2+'%s'+CRL2+'If you %s, icons '+
    'with the same name will occupy precisely the same location. '+
    'We recommend that you rename one of these icons before using '+
    'WinTidy.'+CRL2+'Proceed anyway?';
  sFmtNone = 'There is no saved layout named "%s".';
  sFmtDelNamed = 'Permanently delete the layout named "%s"?';
  sFmtDelRes = 'Permanently delete the %s layout?';
  sFmtOverwrite = 'You have already defined a layout named "%s". '+
    'Do you want to overwrite it?';
  sDatBad1 = 'The WinTidy.Dat file is invalid or corrupted, '+
    'and the backup file WinTidy.bak is absent. You will need to '+
    're-save your layouts.';
  sDatBad3 = 'The WinTidy.Dat file is invalid or corrupted, '+
    'as is the backup file WinTidy.bak. You will need to re-save '+
    'your layouts.';
  sDatBad2 = 'The WinTidy.Dat file is invalid or corrupted. '+
    'WinTidy has loaded the backup, WinTidy.bak, in its place.';
  sVerBad = 'The WinTidy.Dat file was saved under a version of '+
    'WinTidy that uses a different file format. You will need to '+
    're-save your layouts.';
  sPredef = 'This layout precisely matches the predefined layout '+
    'you used last, including the order of the icons. WinTidy can '+
    'store the predefined layout data with your saved layout. When '+
    'you restore the layout, WinTidy will re-apply the predefined '+
    'layout, taking up the slack for any missing icons and adding '+
    'any new ones at the end. Click "Yes" to choose this behavior.'+
    CRL2+'You can choose to omit the predefined layout data from '+
    'this saved layout. Then WinTidy will restore each icon to its '+
    'specified place, omitting any missing ones and warning you '+
    'about any new ones. Click "No" to choose this behavior.'+CRL2+
    'Click "Yes - don''t ask again" to skip this message and '+
    'always save the predefined layout data when there has been no '+
    'change to icon position or order. You can override this '+
    'behavior by moving one icon slightly before saving, or moving '+
    'an item in the icon order tab.'+CRL2+'Store predefined layout '+
    'data with this saved layout?';





  digs = 1;    // digits in the special prefix for icon type
  cTbLr = 00;
  cTbRl = 01;
  cBtLr = 02;
  cBtRl = 03;
  cLrTb = 04;
  cRlTb = 05;
  cLrBt = 06;
  cRlBt = 07;
  cLrSp = 08;
  cTbSp = 09;
  cLrIn = 10;
  cTbIn = 11;
  cIcoAtomName = 'PC Magazine WinTidy Small Icons';
VAR
  DesktopFolder : String;
  CommonFolder  : String;
  ShortExt      : TStringList;

function GetLVHan : HWnd;
function ActiveDesktop : Boolean;

implementation
uses shlobj, allfuncs;

function GetLVHan : HWnd;
// Get the handle of the desktop listview
begin
  Result := FindWindowEx(
              FindWindowEx(
                FindWindowEx(
                  GetDesktopWindow,
                  0, 'Progman', nil),
                0, 'SHELLDLL_DefView', nil),
              0, 'SysListView32', nil);
end;

function ActiveDesktop : Boolean;
begin
  Result := FindWindowEx(
              FindWindowEx(
                FindWindowEx(
                  GetDesktopWindow,
                  0, 'Progman', nil),
                0, 'SHELLDLL_DefView', nil),
              0, 'Internet Explorer_Server', nil) <> 0;
end;

procedure GetShortExt2;
// For speed, use Registry API functions directly rather than
// going thru Delphi TRegistry object.
VAR
  extKey  : HKey;
  namKey  : HKey;
  extIdx  : Integer;
  extStr  : ARRAY[0..MAX_PATH] OF Char;
  extSize : DWORD;
  namStr  : ARRAY[0..MAX_PATH] OF Char;
  namSize : DWORD;
  namType : DWORD;
begin
  extIdx := -1;
  REPEAT
    Inc(extIdx);
    extSize := MAX_PATH;
    IF RegEnumKeyEx(HKEY_CLASSES_ROOT, extIdx, extStr, extSize, nil,
      nil, nil, nil) <> ERROR_SUCCESS THEN Break;
    IF extSize = 0 THEN Continue;
    IF extStr[0] <> '.' THEN Continue;
    IF RegOpenKeyEx(HKEY_CLASSES_ROOT, extStr, 0, KEY_READ,
      extKey) <> ERROR_SUCCESS THEN Continue;
    try
      namSize := MAX_PATH;
      IF RegQueryValueEx(extKey, '', nil, @namType, @namStr[0],
        @namSize) <> ERROR_SUCCESS THEN COntinue;
      IF RegOpenKeyEx(HKEY_CLASSES_ROOT, namStr, 0, KEY_READ,
        namKey) <> ERROR_SUCCESS THEN Continue;
      try
        IF RegQueryValueEx(namKey, 'IsShortcut', nil, nil, nil,
          nil) = ERROR_SUCCESS THEN
            ShortExt.Add(extStr);
      finally
        RegCloseKey(namKey);
      end;
    finally
      RegCloseKey(extKey);
    end;
  UNTIL False;
end;

initialization
  DeskTopFolder := GetSpecialPath(0, CSIDL_DESKTOPDIRECTORY);
  CommonFolder  := GetSpecialPath(0,CSIDL_COMMON_DESKTOPDIRECTORY);
  ShortExt      := TStringList.Create;
  GetShortExt2;
end.
