unit tidythd;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface

uses Windows, Classes, SysUtils;

type
  TTidyThd = class(TThread)
  private
    fNotify : TNotifyEvent; // call this method on change
    fStopEv : THandle;      // stop tracking when event is signaled
  protected
    procedure SyncNotify;
  public
    constructor Create(vNotify : TNotifyEvent; vStopEv : THandle);
    procedure Execute; override;
  published

end;


implementation
uses tidyfuncs;

{ TTidyThd }

constructor TTidyThd.Create(vNotify: TNotifyEvent; vStopEv: THandle);
begin
  Inherited Create(True);
  fNotify := vNotify;
  fStopEv := vStopEv;
end;

procedure TTidyThd.Execute;
VAR
  Hans : ARRAY[0..2] OF THandle;
  H    : THandle;
  NHan : Integer;
begin
  Hans[0] := fStopEv;
  NHan := 1;
  H := FindFirstChangeNotification(PChar(DesktopFolder), False,
    FILE_NOTIFY_CHANGE_FILE_NAME OR FILE_NOTIFY_CHANGE_DIR_NAME);
  // Check returned handle; only track if valid
  IF H <> $FFFFFFFF THEN
    begin
      Hans[nHan] := H;
      Inc(nHan);
    end;
  H := FindFirstChangeNotification(PChar(CommonFolder), False,
    FILE_NOTIFY_CHANGE_FILE_NAME OR FILE_NOTIFY_CHANGE_DIR_NAME);
  // Check returned handle; only track if valid
  IF H <> $FFFFFFFF THEN
    begin
      Hans[nHan] := H;
      Inc(nHan);
    end;
  REPEAT
    CASE WaitForMultipleObjects(nHan, @hans, False, INFINITE) OF
      WAIT_OBJECT_0+1, WAIT_OBJECT_0+2 : begin
        Synchronize(SyncNotify);
      end;
      ELSE Break;
    END;
    // Reset the change notification(s)
    IF nHan > 1 THEN FindNextChangeNotification(Hans[1]);
    IF nHan > 2 THEN FindNextChangeNotification(Hans[2]);
  UNTIL False;
  // Close the change notification(s)
  IF nHan > 1 THEN FindCloseChangeNotification(Hans[1]);
  IF nHan > 2 THEN FindCloseChangeNotification(Hans[2]);
end;

procedure TTidyThd.SyncNotify;
begin
  fNotify(Self);
end;

end.
